(* $Id: godi_versions.mli 678 2007-02-25 17:16:54Z gerd $
 * ----------------------------------------------------------------------
 * Copyright 2003 by Gerd Stolpmann. 
 * 
 * This file is part of godi-tools. 
 *
 * godi-tools is free software; you can redistribute it and/or modify it
 * under the conditions in the file LICENSE.
 *
 * godi-tools is distributed WITHOUT ANY WARRANTY, as detailed in the
 * file LICENSE.
 *)

(* Handle version numbers *)

type condition =
    (* Contains only the version number, not the package name *)
    private
    Exact of string    (* pkg-version or pkg==version *)
  | Any                (* pkg-* *)
  | Ge of string       (* pkg>=version *)
  | Gt of string       (* pkg>version *)
  | Le of string       (* pkg<=version *)
  | Lt of string       (* pkg<version *)
  | Ne of string       (* pkg!=version *)
;;

(* Comparisons of version numbers: 
 *
 * - The package revisions are ignored for the comparisons. E.g. the package
 *   p-3godi1 fulfills Exact "p-3".
 * - There is no fnmatch-style pattern matching, only pkg-* is allowed
 *   to match any version.
 * - The algorithm used here should be the same as in str.c (pkg_install/lib).
 *   I have changed the C algorithm, but there are still differences:
 *    * The C version calls fnmatch if *, ?, [, ] occur in the comparison
 *      string. For maximum compatibility, only "pkg-[0-9]*" should be
 *      used.
 *)


type dependency = string * condition   (* package name, version *)
;;    


val split_name_version : string -> (string * string * int)
  (* Splits a full name into a name, a version number, and a package
   * revision number
   *)

val concat_name_version : string -> string -> int -> string
  (* Concatenates a name, a version, and a package revision to a full name *)

val parse_dependency : string -> dependency 
  (* Parses strings like 'name>=version' and returns the corresponding
   * value
   *)

val print_dependency : dependency -> string
  (* The reverse of parse_dependency *)

val any_dependency : string -> dependency
  (* Returns the dependency (name,Any) *)

val cmp_version_string : string -> string -> int
  (* Compares two version strings. 
   * See version_matches_condition for a description of the sorting
   * order.
   *)

val version_matches_condition : string -> condition -> bool
  (* Whether a version string matches the version condition.
   *
   * version          -> version_part (separator version_part)*
   * separator        -> "." | "+" | "_" | "pl"
   * version_part     -> (decimal_part non_decimal_part)+
   * decimal_part     -> ("0".."9")* preversion_part? 
   * non_decimal_part -> non-decimal string
   * preversion_part  -> "test" | "alpha" | "beta" | "pre" | "rc"
   * 
   * This grammar suggests a sorting order:
   * - versions are sorted lexicographically by the weights of the
   *   version_parts. The separators also have a weight ("." highest
   *   weight, "godi" lowest weight), e.g. "1pl0" < "1.0".
   * - the version_parts are sorted lexicographically by the weights
   *   of their components.
   * - the decimal parts are compared by number. If there is a 
   *   preversion_part, it is sorted before the number, e.g.
   *   "1alpha" < "1". "test" has lowest weight, "rc" highest weight,
   *   e.g. "1test" < "1rc".
   * - for decimal parts:
   *   "" < "test" < "alpha" < "beta" < "pre" < "rc" < "0" 
   * - non-decimal parts are compared by ASCII code
   *
   * Note: This order should closely correlate to the version comparison
   * defined in the C file pkg_install/lib/str.c.
   *
   * TODO: This is not yet the case.
   * Also remove support for "nb" (the "nb" or "godi" suffix should be
   * ignored)
   *)
